@@ -6,7 +6,7 @@ diff.default <- function(x, lag = 1, differences = 1, ...)
     xlen <- if(ismat) dim(x)[1] else length(x)
     if (length(lag) > 1 || length(differences) > 1 ||
         lag < 1 || differences < 1)
-	stop("`lag' and `differences' must be integers >= 1")
+	stop("'lag' and 'differences' must be integers >= 1")
     if (lag * differences >= xlen)
 	return(x[0]) # empty of proper mode
     r <- unclass(x)  # don't want class-specific subset methods
