@@ -29,6 +29,9 @@
 #include <Graphics.h>
 #include <Rdevices.h>
 #include "grDevices.h"
+#ifdef SUPPORT_MBCS
+#include <wchar.h>
+#endif
 
 
 #if defined(__APPLE_CC__) && defined(HAVE_AQUA)
@@ -522,7 +525,7 @@ SEXP Quartz(SEXP args)
 	 free(dev);
 	 error(_("unable to start device Quartz"));
     }
-    gsetVar(install(".Device"), mkString("quartz"), R_NilValue);
+    gsetVar(install(".Device"), mkString("quartz"), R_BaseEnv);
     dd = GEcreateDevDesc(dev);
     addDevice((DevDesc*)dd);
     GEinitDisplayList(dd);
@@ -1499,7 +1502,7 @@ static Rboolean Quartz_Locator(double *x, double *y, NewDevDesc *dd)
     Cursor		arrow ;
     QuartzDesc *xd = (QuartzDesc*)dd->deviceSpecific;
 	int useBeep = asLogical(GetOption(install("locatorBell"), 
-						      R_NilValue));
+						      R_BaseEnv));
 	
     GetPort(&savePort);
 
@@ -1567,15 +1570,25 @@ static void 	Quartz_MetricInfo(int c,
 {
     FMetricRec myFMetric;
     QuartzDesc *xd = (QuartzDesc *) dd-> deviceSpecific;
-    char testo[2];
+    char testo[12];
 	char *ff;
     CGrafPtr savedPort;
     Rect bounds;
     CGPoint position;
 	unsigned char tmp;
 
+#ifdef SUPPORT_MBCS
+    wchar_t wc[2] = L" ";
+    wchar_t *wcs=wc;
+
+    memset(testo,0,sizeof(testo));
+    wc[0] = (unsigned int) c;
+
+    wcsrtombs(testo, (const wchar_t **)&wcs, sizeof(wchar_t), NULL); 
+#else
     testo[0] = c;
     testo[1] = '\0';
+#endif
 /*    fprintf(stderr,"c=%c,>%s<\n",c,testo);
   */  GetPort(&savedPort);
 
