@@ -23,7 +23,7 @@
     ## formal method definition for all but primitives
     setClass("MethodDefinition",
              representation("function", "PossibleMethod",
-                            target = "signature", defined = "signature"),
+                            target = "signature", defined = "signature", generic = "character"),
              where = envir); clList <- c(clList, "MethodDefinition")
     ## class for default methods made from ordinary functions
     setClass("derivedDefaultMethod", "MethodDefinition")
@@ -97,28 +97,39 @@
                   assign(".nextMethod", method@nextMethod, envir = envir)
                   method
               }, where = envir)
-    setGeneric("addNextMethod", function(method, f = "<unknown>", mlist, optional = FALSE, envir)
+    setGeneric("addNextMethod", function(method, f = "<unknown>",
+                                         mlist = getMethods(f), optional = FALSE, envir)
                standardGeneric("addNextMethod"), where = envir)
     setMethod("addNextMethod", "MethodDefinition",
               function(method, f, mlist, optional, envir) {
+                if(.UsingMethodsTables())
+                  value <- .findNextFromTable(method, f, optional, envir)
+                else {
                   value <- .findNextMethod(method, f, mlist, optional, list(method@defined), envir)
                   new("MethodWithNext", method, nextMethod = value,
                       excluded = list(method@defined))
+                }
               }, where = envir)
     setMethod("addNextMethod", "MethodWithNext",
               function(method, f, mlist, optional, envir) {
+                if(.UsingMethodsTables())
+                  .findNextFromTable(method, f, optional, envir, method@excluded)
+                else {
                   excluded <- c(method@excluded, list(method@defined))
                   value <- .findNextMethod(method, f, mlist, optional, excluded, envir)
                   new("MethodWithNext", method, nextMethod = value,
                       excluded = excluded)
+                }
               }, where = envir)
     .initGeneric <- function(.Object, ...) {
             value <- standardGeneric("initialize")
             if(!identical(class(value), class(.Object))) {
                 cv <- class(value)
                 co <- class(.Object)
-                if(.identC(cv, co) && is.null(packageSlot(cv))) {
-                    if(is.na(match(cv, .BasicClasses))) {
+                if(.identC(cv[[1]], co)) {
+                  ## ignore S3 with multiple classes  or basic classes
+                    if(is.na(match(cv, .BasicClasses)) &&
+                       length(cv) == 1) {
                         warning(gettextf("missing package slot (%s) in object of class \"%s\" (package info added)", packageSlot(co), class(.Object)),
                                 domain = NA)
                         class(value) <- class(.Object)
@@ -128,7 +139,7 @@
                 }
                 else
                     stop(gettextf("initialize method returned an object of class \"%s\" instead of the required class \"%s\"",
-                                  class(value), class(.Object)), domain = NA)
+                                  paste(class(value), collapse=", "), class(.Object)), domain = NA)
             }
             value
         }
@@ -176,7 +187,7 @@
                   }
                   else {
                       mat <- rbind(data, pkg)
-		      dimnames(mat) <- list(c("Object:", "From:"),
+		      dimnames(mat) <- list(c("Object:", "Package:"),
 					    rep("", length(data)))
                       show(mat)
                   }
@@ -198,6 +209,14 @@
     setMethod("rbind2", signature(x = "ANY", y = "missing"),
 	      function(x,y) .Internal(rbind(deparse.level = 0, x)))
 
+    ## a show() method for the signature class
+    setMethod("show", "signature", function(object) {
+      message("An object of class \"", class(object), "\"")
+      val <- object@.Data
+      names(val) <- object@names
+      callNextMethod(val)
+    } )
+
 ### Uncomment next line if we want special initialize methods for basic classes
 ###    .InitBasicClassMethods(where)
 }
@@ -216,7 +235,7 @@
                 formalNames <- formalNames[-dots]
         }
         if(is.null(sigArgs))
-            names(signature) <- formalNames[seq(along = classes)]
+            names(signature) <- formalNames[seq_along(classes)]
         else if(length(sigArgs) > 0 && any(is.na(match(sigArgs, formalNames))))
             stop(gettextf("the names in signature for method (%s) do not match function's arguments (%s)",
                           paste(sigArgs, collapse = ", "),
