@@ -87,6 +87,7 @@ SEXP setup_starma(SEXP na, SEXP x, SEXP pn, SEXP xreg, SEXP pm,
     Starma G;
     int i, n, m, ip, iq, ir, np;
     SEXP res;
+    double *rx = REAL(x), *rxreg = REAL(xreg);
 
     G = Calloc(1, starma_struct);
     G->mp = INTEGER(na)[0];
@@ -118,8 +119,8 @@ SEXP setup_starma(SEXP na, SEXP x, SEXP pn, SEXP xreg, SEXP pm,
     G->theta = Calloc(ir, double);
     G->reg = Calloc(1 + n*m, double); /* AIX can't calloc 0 items */
     G->delta = asReal(dt);
-    for(i = 0; i < n; i++) G->w[i] = G->wkeep[i] = REAL(x)[i];
-    for(i = 0; i < n*m; i++) G->reg[i] = REAL(xreg)[i];
+    for(i = 0; i < n; i++) G->w[i] = G->wkeep[i] = rx[i];
+    for(i = 0; i < n*m; i++) G->reg[i] = rxreg[i];
     Starma_tag = install("STARMA_TAG");
     res = R_MakeExternalPtr(G, Starma_tag, R_NilValue);
     return res;
@@ -250,10 +251,12 @@ SEXP get_resid(SEXP pG)
 {
     SEXP res;
     int i;
+    double *rres;
     GET_STARMA;
 
     res = allocVector(REALSXP, G->n);
-    for(i = 0; i < G->n; i++) REAL(res)[i] = G->resid[i];
+    rres = REAL(res);
+    for(i = 0; i < G->n; i++) rres[i] = G->resid[i];
     return res;
 }
 
