@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2003   The R Development Core Team.
+ *  Copyright (C) 2003-6   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 #include <R.h>
@@ -30,6 +30,7 @@ void Renctest(char **x)
 static const R_CallMethodDef callMethods[] = {
     {"delim_match", (DL_FUNC) &delim_match, 2},
     {"Rmd5", (DL_FUNC) &Rmd5, 1},
+    {"check_nonASCII", (DL_FUNC) &check_nonASCII, 2},
     {NULL, NULL, 0}
 };
 
@@ -39,7 +40,11 @@ static const R_CMethodDef CEntries[]  = {
 };
 
 
-void R_init_tools(DllInfo *dll)
+void
+#ifdef HAVE_VISIBILITY_ATTRIBUTE
+__attribute__ ((visibility ("default")))
+#endif
+R_init_tools(DllInfo *dll)
 {
     R_registerRoutines(dll, CEntries, callMethods, NULL, NULL);
     R_useDynamicSymbols(dll, FALSE);
