@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8> char here is either ASCII or handled as a whole */
@@ -640,10 +640,6 @@ static SEGP* contourLines(double *x, int nx, double *y, int ny,
     return segmentDB;
 }
 
-/* maximal number of line segments of one contour segment:
- * for preventing infinite loops -- shouldn't be needed --> warning */
-#define MAX_ns 25000
-
 #define CONTOUR_LIST_STEP 100
 #define CONTOUR_LIST_LEVEL 0
 #define CONTOUR_LIST_X 1
@@ -664,6 +660,7 @@ static SEXP growList(SEXP oldlist) {
  * Store the list of segments for a single level in the SEXP
  * list that will be returned to the user
  */
+static
 int addContourLines(double *x, int nx, double *y, int ny,
 		     double *z, double zc, double atom,
 		     SEGP* segmentDB, int nlines, SEXP container)
@@ -715,12 +712,12 @@ int addContourLines(double *x, int nx, double *y, int ny,
 		/* ns := #{segments of polyline} -- need to allocate */
 		s = start;
 		ns = 0;
-		/* MAX_ns: prevent inf.loop (shouldn't be needed) */
-		while (s && ns < MAX_ns) {
+		/* max_contour_segments: prevent inf.loop (shouldn't be needed) */
+		while (s && ns < max_contour_segments) {
 		    ns++;
 		    s = s->next;
 		}
-		if(ns == MAX_ns)
+		if(ns == max_contour_segments)
 		    warning(_("contour(): circular/long seglist -- bug.report()!"));
 
 		/* countour midpoint : use for labelling sometime (not yet!) */
@@ -739,7 +736,7 @@ int addContourLines(double *x, int nx, double *y, int ny,
 		REAL(xsxp)[0] = s->x0;
 		REAL(ysxp)[0] = s->y0;
 		ns = 1;
-		while (s->next && ns < MAX_ns) {
+		while (s->next && ns < max_contour_segments) {
 		    s = s->next;
 		    REAL(xsxp)[ns] = s->x0;
 		    REAL(ysxp)[ns++] = s->y0;
@@ -782,8 +779,7 @@ int addContourLines(double *x, int nx, double *y, int ny,
  *     sub-list = x vector, y vector, and cut-value.
  */
 SEXP GEcontourLines(double *x, int nx, double *y, int ny,
-		    double *z, double *levels, int nl,
-		    GEDevDesc *dd)
+		    double *z, double *levels, int nl)
 {
     char *vmax;
     int i, nlines, len;
@@ -811,7 +807,7 @@ SEXP GEcontourLines(double *x, int nx, double *y, int ny,
     /* change to 1e-3, reconsidered because of PR#897
      * but 1e-7, and even  2*DBL_EPSILON do not prevent inf.loop in contour().
      * maybe something like   16 * DBL_EPSILON * (..).
-     * see also MAX_ns above */
+     * see also max_contour_segments above */
     atom = 1e-3 * (zmax - zmin);
     /*
      * Create a "container" which is a list with only 1 element.
@@ -872,11 +868,10 @@ SEXP GEcontourLines(double *x, int nx, double *y, int ny,
 
 SEXP GEdrawContourLines();
 
-SEXP do_contourLines(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_contourLines(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP oargs, c, x, y, z;
     int nx, ny, nc;
-    GEDevDesc *dd = GEcurrentDevice();
 
     oargs = args;
 
@@ -900,7 +895,7 @@ SEXP do_contourLines(SEXP call, SEXP op, SEXP args, SEXP env)
     nc = LENGTH(c);
     args = CDR(args);
 
-    return GEcontourLines(REAL(x), nx, REAL(y), ny, REAL(z), REAL(c), nc, dd);
+    return GEcontourLines(REAL(x), nx, REAL(y), ny, REAL(z), REAL(c), nc);
 }
 
 /*
@@ -1010,12 +1005,12 @@ static void contour(SEXP x, int nx, SEXP y, int ny, SEXP z,
 	    /* ns := #{segments of polyline} -- need to allocate */
 	    s = start;
 	    ns = 0;
-	    /* MAX_ns: prevent inf.loop (shouldn't be needed) */
-	    while (s && ns < MAX_ns) {
+	    /* max_contour_segments: prevent inf.loop (shouldn't be needed) */
+	    while (s && ns < max_contour_segments) {
 		ns++;
 		s = s->next;
 	    }
-	    if(ns == MAX_ns)
+	    if(ns == max_contour_segments)
 		warning(_("contour(): circular/long seglist -- bug.report()!"));
 
 	    /* countour midpoint : use for labelling sometime (not yet!) */
@@ -1029,7 +1024,7 @@ static void contour(SEXP x, int nx, SEXP y, int ny, SEXP z,
 	    ns = 0;
 	    xxx[ns] = s->x0;
 	    yyy[ns++] = s->y0;
-	    while (s->next && ns < MAX_ns) {
+	    while (s->next && ns < max_contour_segments) {
 		s = s->next;
 		xxx[ns] = s->x0;
 		yyy[ns++] = s->y0;
@@ -1360,7 +1355,7 @@ static void contour(SEXP x, int nx, SEXP y, int ny, SEXP z,
 /* contour(x, y, z, levels, labels, labcex, drawlabels,
  *         method, vfont, col = col, lty = lty, lwd = lwd)
  */
-SEXP do_contour(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_contour(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP oargs, c, x, y, z, vfont, col, rawcol, lty, lwd, labels;
     int i, j, nx, ny, nc, ncol, nlty, nlwd;
@@ -1417,7 +1412,7 @@ SEXP do_contour(SEXP call, SEXP op, SEXP args, SEXP env)
 
     method = asInteger(CAR(args)); args = CDR(args);
     if (method < 1 || method > 3)
-	errorcall(call, _("invalid value for 'method'"));
+	errorcall(call, _("invalid '%s' value"), "method");
 
     PROTECT(vfont = FixupVFont(CAR(args)));
     if (!isNull(vfont)) {
@@ -1480,13 +1475,14 @@ SEXP do_contour(SEXP call, SEXP op, SEXP args, SEXP env)
 	    warning(_("all z values are equal"));
 	else
 	    warning(_("all z values are NA"));
+        UNPROTECT(4);
 	return R_NilValue;
     }
 
     /* change to 1e-3, reconsidered because of PR#897
      * but 1e-7, and even  2*DBL_EPSILON do not prevent inf.loop in contour().
      * maybe something like   16 * DBL_EPSILON * (..).
-     * see also MAX_ns above */
+     * see also max_contour_segments above */
     atom = 1e-3 * (zmax - zmin);
 
     /* Initialize the segment data base */
@@ -1658,7 +1654,7 @@ FindPolygonVertices(double low, double high,
 */
 
 /* filledcontour(x, y, z, levels, col) */
-SEXP do_filledcontour(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_filledcontour(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP oargs, sx, sy, sz, sc, scol;
     double *x, *y, *z, *c;
@@ -1775,7 +1771,7 @@ SEXP do_filledcontour(SEXP call, SEXP op, SEXP args, SEXP env)
 
 
 /* image(x, y, z, col, breaks) */
-SEXP do_image(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_image(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP oargs, sx, sy, sz, sc;
     double *x, *y;
@@ -1813,16 +1809,7 @@ SEXP do_image(SEXP call, SEXP op, SEXP args, SEXP env)
     z = INTEGER(sz);
     c = (unsigned*)INTEGER(sc);
 
-    /* Check of grid coordinates */
-    /* We want them to all be finite and in strictly ascending order */
-
-    if (nx < 1 || ny < 1) goto badxy;
-    if (!R_FINITE(x[0])) goto badxy;
-    if (!R_FINITE(y[0])) goto badxy;
-    for (i = 1; i < nx; i++)
-	if (!R_FINITE(x[i]) || x[i] <= x[i - 1]) goto badxy;
-    for (j = 1; j < ny; j++)
-	if (!R_FINITE(y[j]) || y[j] <= y[j - 1]) goto badxy;
+    /* Check of grid coordinates now done in C code */
 
     colsave = Rf_gpptr(dd)->col;
     xpdsave = Rf_gpptr(dd)->xpd;
@@ -1847,10 +1834,6 @@ SEXP do_image(SEXP call, SEXP op, SEXP args, SEXP env)
     if (GRecording(call, dd))
 	recordGraphicOperation(op, oargs, dd);
     return R_NilValue;
-
-  badxy:
-    errorcall(call, _("invalid x / y values or limits"));
-    return R_NilValue;/* never used; to keep -Wall happy */
 }
 
 	/*  P e r s p e c t i v e   S u r f a c e   P l o t s  */
@@ -2457,14 +2440,14 @@ static void PerspAxes(double *x, double *y, double *z,
     Rf_gpptr(dd)->xpd = xpdsave;
 }
 
-SEXP do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, y, z, xlim, ylim, zlim;
     SEXP depth, indx, originalArgs;
     SEXP col, border, xlab, ylab, zlab;
     double theta, phi, r, d;
     double ltheta, lphi;
-    double expand, xc, yc, zc, xs, ys, zs;
+    double expand, xc = 0.0, yc = 0.0, zc = 0.0, xs = 0.0, ys = 0.0, zs = 0.0;
     int i, j, scale, ncol, dobox, doaxes, nTicks, tickType;
     DevDesc *dd;
 
@@ -2474,28 +2457,28 @@ SEXP do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
     originalArgs = args;
 
     PROTECT(x = coerceVector(CAR(args), REALSXP));
-    if (length(x) < 2) errorcall(call, _("invalid 'x' argument"));
+    if (length(x) < 2) errorcall(call, _("invalid '%s' argument"), "x");
     args = CDR(args);
 
     PROTECT(y = coerceVector(CAR(args), REALSXP));
-    if (length(y) < 2) errorcall(call, _("invalid 'y' argument"));
+    if (length(y) < 2) errorcall(call, _("invalid '%s' argument"), "x");
     args = CDR(args);
 
     PROTECT(z = coerceVector(CAR(args), REALSXP));
     if (!isMatrix(z) || nrows(z) != length(x) || ncols(z) != length(y))
-	errorcall(call, _("invalid 'z' argument"));
+	errorcall(call, _("invalid '%s' argument"), "z");
     args = CDR(args);
 
     PROTECT(xlim = coerceVector(CAR(args), REALSXP));
-    if (length(xlim) != 2) errorcall(call, _("invalid 'xlim' argument"));
+    if (length(xlim) != 2) errorcall(call, _("invalid '%s' argument"), "xlim");
     args = CDR(args);
 
     PROTECT(ylim = coerceVector(CAR(args), REALSXP));
-    if (length(ylim) != 2) errorcall(call, _("invalid 'ylim' argument"));
+    if (length(ylim) != 2) errorcall(call, _("invalid '%s' argument"), "ylim");
     args = CDR(args);
 
     PROTECT(zlim = coerceVector(CAR(args), REALSXP));
-    if (length(zlim) != 2) errorcall(call, _("invalid 'zlim' argument"));
+    if (length(zlim) != 2) errorcall(call, _("invalid '%s' argument"), "zlim");
     args = CDR(args);
 
     /* Checks on x/y/z Limits */
@@ -2552,13 +2535,13 @@ SEXP do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
 	d < 0 || r < 0)
 	errorcall(call, _("invalid viewing parameters"));
     if (!R_FINITE(expand) || expand < 0)
-	errorcall(call, _("invalid 'expand' value"));
+	errorcall(call, _("invalid '%s' value"), "expand");
     if (scale == NA_LOGICAL)
 	scale = 0;
     if ((nTicks == NA_INTEGER) || (nTicks < 0))
-	errorcall(call, _("invalid 'nticks' value"));
+	errorcall(call, _("invalid '%s' value"), "nticks");
     if ((tickType == NA_INTEGER) || (tickType < 1) || (tickType > 2))
-	errorcall(call, _("invalid 'ticktype' value"));
+	errorcall(call, _("invalid '%s' value"), "ticktype");
 
     dd = CurrentDevice();
 
@@ -2566,10 +2549,11 @@ SEXP do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
 
     PROTECT(col = FixupCol(col, Rf_gpptr(dd)->bg));
     ncol = LENGTH(col);
-    if (ncol < 1) errorcall(call, _("invalid 'col' specification"));
+    if (ncol < 1) errorcall(call, _("invalid '%s' specification"), "col");
     if(!R_OPAQUE(INTEGER(col)[0])) DoLighting = FALSE;
     PROTECT(border = FixupCol(border, Rf_gpptr(dd)->fg));
-    if (length(border) < 1) errorcall(call, _("invalid 'border' specification"));
+    if (length(border) < 1) 
+	errorcall(call, _("invalid '%s' specification"), "border");
 
     GSetState(1, dd);
     GSavePars(dd);
