@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2005  R Development Core Team
+ *  Copyright (C) 1997--2006  R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8> char here is handled as a whole.
@@ -29,6 +29,7 @@
 
 #include <Defn.h>
 #include <Rmath.h>
+#include <errno.h>
 
 static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2);
 static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2);
@@ -36,7 +37,7 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call);
 static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2);
 static SEXP raw_relop(RELOP_TYPE code, SEXP s1, SEXP s2);
 
-SEXP do_relop(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_relop(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
 
@@ -46,7 +47,7 @@ SEXP do_relop(SEXP call, SEXP op, SEXP args, SEXP env)
     return do_relop_dflt(call, op, CAR(args), CADR(args));
 }
 
-SEXP do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
+SEXP attribute_hidden do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
 {
     SEXP class=R_NilValue, dims, tsp=R_NilValue, xnames, ynames;
     int nx, ny, xarray, yarray, xts, yts;
@@ -445,9 +446,12 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 
 #endif
 
+
+/* POSIX allows EINVAL when one of the strings contains characters
+   outside the collation domain. */
 static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 {
-    int i, n, n1, n2;
+    int i, n, n1, n2, res;
     SEXP ans;
 
     n1 = LENGTH(s1);
@@ -489,12 +493,15 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else
-	    if (STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
-		       CHAR(STRING_ELT(s2, i % n2))) < 0)
-		LOGICAL(ans)[i] = 1;
-	    else
-		LOGICAL(ans)[i] = 0;
+ 	    else {
+		errno = 0;
+		res = STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
+			      CHAR(STRING_ELT(s2, i % n2)));
+		if(errno)  
+		    LOGICAL(ans)[i] = NA_LOGICAL;
+		else
+		    LOGICAL(ans)[i] = (res < 0)?1:0;
+	    }
 	}
 	break;
     case GTOP:
@@ -502,12 +509,15 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else
-	    if (STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
-		       CHAR(STRING_ELT(s2, i % n2))) > 0)
-		LOGICAL(ans)[i] = 1;
-	    else
-		LOGICAL(ans)[i] = 0;
+ 	    else {
+		errno = 0;
+		res = STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
+			      CHAR(STRING_ELT(s2, i % n2)));
+		if(errno)  
+		    LOGICAL(ans)[i] = NA_LOGICAL;
+		else
+		    LOGICAL(ans)[i] = (res > 0)?1:0;
+	    }
 	}
 	break;
     case LEOP:
@@ -515,12 +525,15 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else
-	    if (STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
-		       CHAR(STRING_ELT(s2, i % n2))) <= 0)
-		LOGICAL(ans)[i] = 1;
-	    else
-		LOGICAL(ans)[i] = 0;
+ 	    else {
+		errno = 0;
+		res = STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
+			      CHAR(STRING_ELT(s2, i % n2)));
+		if(errno)  
+		    LOGICAL(ans)[i] = NA_LOGICAL;
+		else
+		    LOGICAL(ans)[i] = (res <= 0)?1:0;
+	    }
 	}
 	break;
     case GEOP:
@@ -528,12 +541,15 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 	    if ((STRING_ELT(s1, i % n1) == NA_STRING) ||
 		(STRING_ELT(s2, i % n2) == NA_STRING))
 		LOGICAL(ans)[i] = NA_LOGICAL;
- 	    else
-	    if (STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
-		       CHAR(STRING_ELT(s2, i % n2))) >= 0)
-		LOGICAL(ans)[i] = 1;
-	    else
-		LOGICAL(ans)[i] = 0;
+  	    else {
+		errno = 0;
+		res = STRCOLL(CHAR(STRING_ELT(s1, i % n1)),
+			      CHAR(STRING_ELT(s2, i % n2)));
+		if(errno)  
+		    LOGICAL(ans)[i] = NA_LOGICAL;
+		else
+		    LOGICAL(ans)[i] = (res >= 0)?1:0;
+	    }
 	}
 	break;
     }
