@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
-  *  Copyright (C) 2001-5 The R Development Core Team
+  *  Copyright (C) 2001-6 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8> char here is either ASCII or handled as a whole.
@@ -29,13 +29,6 @@
 #include <Rdynpriv.h>
 #include <Graphics.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-#if !HAVE_DECL_ALLOCA
-extern char *alloca(size_t);
-#endif
-
 typedef struct {
     R_GE_VTextRoutine GEVText;
     R_GE_VStrWidthRoutine GEVStrWidth;
@@ -71,7 +64,7 @@ R_GE_setVFontRoutines(R_GE_VStrWidthRoutine vwidth,
 
 static void vfonts_Init(void)
 {
-    int res = moduleCdynload("vfonts", 1, 1);
+    int res = R_moduleCdynload("vfonts", 1, 1);
     initialized = -1;
     if(!res) return;
     if(!ptr->GEVStrWidth)
@@ -87,13 +80,16 @@ double GVStrWidth (const unsigned char *s, int typeface, int fontindex,
     char *str = (char *)s;
 #ifdef SUPPORT_MBCS
     char *buff;
+    Rboolean conv = mbcslocale;
 #endif
     gcontextFromGP(&gc, dd);
     gc.fontface = typeface;
     gc.fontfamily[0] = fontindex;
 #ifdef SUPPORT_MBCS
-    if(mbcslocale && !utf8strIsASCII(str)) {
+    if(typeface == 0  && (fontindex == 5 || fontindex == 6)) conv = FALSE;
+    if(conv && !utf8strIsASCII(str)) {
 	buff = alloca(strlen(str)+1); /* Output string cannot be longer */
+        R_CheckStack();
 	if(!buff) error(_("allocation failure in GVStrWidth"));
 	mbcsToLatin1((char*) s, buff);
 	str = buff;
@@ -124,13 +120,16 @@ double GVStrHeight (const unsigned char *s, int typeface, int fontindex,
     char *str = (char *)s;
 #ifdef SUPPORT_MBCS
     char *buff;
+    Rboolean conv = mbcslocale;
 #endif
     gcontextFromGP(&gc, dd);
     gc.fontface = typeface;
     gc.fontfamily[0] = fontindex;
 #ifdef SUPPORT_MBCS
-    if(mbcslocale && !utf8strIsASCII(str)) {
+    if(typeface == 0  && (fontindex == 5 || fontindex == 6)) conv = FALSE;
+    if(conv && !utf8strIsASCII(str)) {
 	buff = alloca(strlen(str)+1); /* Output string cannot be longer */
+        R_CheckStack();
 	if(!buff) error(_("allocation failure in GVStrHeight"));
 	mbcsToLatin1((char *) s, buff);
 	str = buff;
@@ -163,6 +162,7 @@ void GVText (double x, double y, int unit, char *s,
     char *str = s;
 #ifdef SUPPORT_MBCS
     char *buff;
+    Rboolean conv = mbcslocale;
 #endif
     gcontextFromGP(&gc, dd);
     /*
@@ -173,8 +173,10 @@ void GVText (double x, double y, int unit, char *s,
     gc.fontface = fontindex;
     gc.fontfamily[0] = typeface;
 #ifdef SUPPORT_MBCS
-    if(mbcslocale && !utf8strIsASCII(str)) {
+    if(typeface == 0  && (fontindex == 5 || fontindex == 6)) conv = FALSE;
+    if(conv && !utf8strIsASCII(str)) {
 	buff = alloca(strlen(str)+1); /* Output string cannot be longer */
+	R_CheckStack();
 	if(!buff) error(_("allocation failure in GVText"));
 	mbcsToLatin1(s, buff);
 	str = buff;
